/*
 * Copyright (c) 1988-1997 Sam Leffler
 * Copyright (c) 1991-1997 Silicon Graphics, Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the names of
 * Sam Leffler and Silicon Graphics may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of Sam Leffler and Silicon Graphics.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL SAM LEFFLER OR SILICON GRAPHICS BE LIABLE FOR
 * ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
 * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF
 * LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
 * OF THIS SOFTWARE.
 */

#ifndef _TIFFIO_
#define	_TIFFIO_

/*
 * TIFF I/O Library Definitions.
 */
#include "tiff.h"
#include "tiffvers.h"

/*
 * TIFF is defined as an incomplete type to hide the
 * library's internal data structures from clients.
 */
typedef struct tiff TIFF;

/*
 * The following typedefs define the intrinsic size of
 * data types used in the *exported* interfaces.  These
 * definitions depend on the proper definition of types
 * in tiff.h.  Note also that the varargs interface used
 * to pass tag types and values uses the types defined in
 * tiff.h directly.
 *
 * NB: ttag_t is unsigned int and not unsigned short because
 *     ANSI C requires that the type before the ellipsis be a
 *     promoted type (i.e. one of int, unsigned int, pointer,
 *     or double) and because we defined pseudo-tags that are
 *     outside the range of legal Aldus-assigned tags.
 * NB: tsize_t is signed and not unsigned because some functions
 *     return -1.
 * NB: toff_t is not off_t for many reasons; TIFFs max out at
 *     32-bit file offsets, and BigTIFF maxes out at 64-bit
 *     offsets being the most important, and to ensure use of
 *     a consistently unsigned type across architectures.
 *     Prior to libtiff 4.0, this was an unsigned 32 bit type.
 */
/*
 * this is the machine addressing size type, only it's signed, so make it
 * int32 on 32bit machines, int64 on 64bit machines
 */
typedef TIFF_SSIZE_T tmsize_t;
typedef uint64 toff_t;          /* file offset */
/* the following are deprecated and should be replaced by their defining
   counterparts */
typedef uint32 ttag_t;          /* directory tag */
typedef uint16 tdir_t;          /* directory index */
typedef uint16 tsample_t;       /* sample number */
typedef uint32 tstrile_t;       /* strip or tile number */
typedef tstrile_t tstrip_t;     /* strip number */
typedef tstrile_t ttile_t;      /* tile number */
typedef tmsize_t tsize_t;       /* i/o size in bytes */
typedef void* tdata_t;          /* image data ref */

#define TIFF_SIZEOF_DOUBLE 8

#if !defined(__WIN32__) && (defined(_WIN32) || defined(WIN32))
#define __WIN32__
#endif

/*
 * On windows you should define USE_WIN32_FILEIO if you are using tif_win32.c
 * or AVOID_WIN32_FILEIO if you are using something else (like tif_unix.c).
 *
 * By default tif_unix.c is assumed.
 */

#if defined(_WINDOWS) || defined(__WIN32__) || defined(_Windows)
#  if !defined(__CYGWIN) && !defined(AVOID_WIN32_FILEIO) && !defined(USE_WIN32_FILEIO)
#    define AVOID_WIN32_FILEIO
#  endif
#endif

#if defined(USE_WIN32_FILEIO)
# define VC_EXTRALEAN
# include <windows.h>
# ifdef __WIN32__
DECLARE_HANDLE(thandle_t);     /* Win32 file handle */
# else
typedef HFILE thandle_t;       /* client data handle */
# endif /* __WIN32__ */
#else
typedef void* thandle_t;       /* client data handle */
#endif /* USE_WIN32_FILEIO */

/*
 * Flags to pass to TIFFPrintDirectory to control
 * printing of data structures that are potentially
 * very large.   Bit-or these flags to enable printing
 * multiple items.
 */
#define TIFFPRINT_NONE	       0x0    /* no extra info */
#define TIFFPRINT_STRIPS       0x1    /* strips/tiles info */
#define TIFFPRINT_CURVES       0x2    /* color/gray response curves */
#define TIFFPRINT_COLORMAP     0x4    /* colormap */
#define TIFFPRINT_JPEGQTABLES  0x100  /* JPEG Q matrices */
#define TIFFPRINT_JPEGACTABLES 0x200  /* JPEG AC tables */
#define TIFFPRINT_JPEGDCTABLES 0x200  /* JPEG DC tables */

/* 
 * Colour conversion stuff
 */

/* reference white */
#define D65_X0 (95.0470F)
#define D65_Y0 (100.0F)
#define D65_Z0 (108.8827F)

#define D50_X0 (96.4250F)
#define D50_Y0 (100.0F)
#define D50_Z0 (82.4680F)

/* Structure for holding information about a display device. */

typedef unsigned char TIFFRGBValue;               /* 8-bit samples */

typedef struct {
	float d_mat[3][3];                        /* XYZ -> luminance matrix */
	float d_YCR;                              /* Light o/p for reference white */
	float d_YCG;
	float d_YCB;
	uint32 d_Vrwr;                            /* Pixel values for ref. white */
	uint32 d_Vrwg;
	uint32 d_Vrwb;
	float d_Y0R;                              /* Residual light for black pixel */
	float d_Y0G;
	float d_Y0B;
	float d_gammaR;                           /* Gamma values for the three guns */
	float d_gammaG;
	float d_gammaB;
} TIFFDisplay;

typedef struct {                                  /* YCbCr->RGB support */
	TIFFRGBValue* clamptab;                   /* range clamping table */
	int32* Cr_r_tab;
	int32* Cb_b_tab;
	int32* Cr_g_tab;
	int32* Cb_g_tab;
	int32* Y_tab;
} TIFFYCbCrToRGB;

typedef struct {                                  /* CIE Lab 1976->RGB support */
	tiff_int_t range;                             /* Size of conversion table */
#define CIELABTORGB_TABLE_RANGE 1500
	float rstep, gstep, bstep;
	float X0, Y0, Z0;                         /* Reference white point */
	TIFFDisplay display;
	float Yr2r[CIELABTORGB_TABLE_RANGE + 1];  /* Conversion of Yr to r */
	float Yg2g[CIELABTORGB_TABLE_RANGE + 1];  /* Conversion of Yg to g */
	float Yb2b[CIELABTORGB_TABLE_RANGE + 1];  /* Conversion of Yb to b */
} TIFFCIELabToRGB;

/*
 * RGBA-style image support.
 */
typedef struct _TIFFRGBAImage TIFFRGBAImage;
/*
 * The image reading and conversion routines invoke
 * ``put routines'' to copy/image/whatever tiles of
 * raw image data.  A default set of routines are 
 * provided to convert/copy raw image data to 8-bit
 * packed ABGR format rasters.  Applications can supply
 * alternate routines that unpack the data into a
 * different format or, for example, unpack the data
 * and draw the unpacked raster on the display.
 */
typedef void TIFF_CALLBACK (*tileContigRoutine) (TIFFRGBAImage*, uint32*, uint32, uint32, uint32, uint32, int32, int32, unsigned char*);
typedef void TIFF_CALLBACK (*tileSeparateRoutine)
    (TIFFRGBAImage*, uint32*, uint32, uint32, uint32, uint32, int32, int32,
	unsigned char*, unsigned char*, unsigned char*, unsigned char*);
/*
 * RGBA-reader state.
 */
struct _TIFFRGBAImage {
	TIFF* tif;                              /* image handle */
	tiff_int_t stoponerr;                          /* stop on read error */
	tiff_int_t isContig;                           /* data is packed/separate */
	tiff_int_t alpha;                              /* type of alpha data present */
	uint32 width;                           /* image width */
	uint32 height;                          /* image height */
	uint16 bitspersample;                   /* image bits/sample */
	uint16 samplesperpixel;                 /* image samples/pixel */
	uint16 orientation;                     /* image orientation */
	uint16 req_orientation;                 /* requested orientation */
	uint16 photometric;                     /* image photometric interp */
	uint16* redcmap;                        /* colormap palette */
	uint16* greencmap;
	uint16* bluecmap;
	/* get image data routine */
	tiff_int_t TIFF_CALLBACK (*get)(TIFFRGBAImage*, uint32*, uint32, uint32);
	/* put decoded strip/tile */
	union {
	    void TIFF_CALLBACK (*any)(TIFFRGBAImage*);
	    tileContigRoutine contig;
	    tileSeparateRoutine separate;
	} put;
	TIFFRGBValue* Map;                      /* sample mapping array */
	uint32** BWmap;                         /* black&white map */
	uint32** PALmap;                        /* palette image map */
	TIFFYCbCrToRGB* ycbcr;                  /* YCbCr conversion state */
	TIFFCIELabToRGB* cielab;                /* CIE L*a*b conversion state */

	uint8* UaToAa;                          /* Unassociated alpha to associated alpha conversion LUT */
	uint8* Bitdepth16To8;                   /* LUT for conversion from 16bit to 8bit values */

	int32 row_offset;
	int32 col_offset;
};

/*
 * Macros for extracting components from the
 * packed ABGR form returned by TIFFReadRGBAImage.
 */
#define TIFFGetR(abgr) ((abgr) & 0xff)
#define TIFFGetG(abgr) (((abgr) >> 8) & 0xff)
#define TIFFGetB(abgr) (((abgr) >> 16) & 0xff)
#define TIFFGetA(abgr) (((abgr) >> 24) & 0xff)

/*
 * A CODEC is a software package that implements decoding,
 * encoding, or decoding+encoding of a compression algorithm.
 * The library provides a collection of builtin codecs.
 * More codecs may be registered through calls to the library
 * and/or the builtin implementations may be overridden.
 */
typedef tiff_int_t TIFF_CALLBACK (*TIFFInitMethod)(TIFF*, tiff_uint_t);
typedef struct {
	char* name;
	uint16 scheme;
	TIFFInitMethod init;
} TIFFCodec;

#include <stdio.h>
#include <stdarg.h>

/* share internal LogLuv conversion routines? */
#ifndef LOGLUV_PUBLIC
#define LOGLUV_PUBLIC 1
#endif

#if !defined(__GNUC__) && !defined(__attribute__)
#  define __attribute__(x) /*nothing*/
#endif

#if defined(c_plusplus) || defined(__cplusplus)
extern "C" {
#endif

typedef void TIFF_CALLBACK (*TIFFErrorHandler)(const char*, const char*, va_list);
typedef void TIFF_CALLBACK (*TIFFErrorHandlerExt)(thandle_t, const char*, const char*, va_list);
typedef tmsize_t TIFF_CALLBACK (*TIFFReadWriteProc)(thandle_t, void*, tmsize_t);
typedef toff_t TIFF_CALLBACK (*TIFFSeekProc)(thandle_t, toff_t, tiff_int_t);
typedef tiff_int_t TIFF_CALLBACK (*TIFFCloseProc)(thandle_t);
typedef toff_t TIFF_CALLBACK (*TIFFSizeProc)(thandle_t);
typedef tiff_int_t TIFF_CALLBACK (*TIFFMapFileProc)(thandle_t, void** base, toff_t* size);
typedef void TIFF_CALLBACK (*TIFFUnmapFileProc)(thandle_t, void* base, toff_t size);
typedef void TIFF_CALLBACK (*TIFFExtendProc)(TIFF*);

const char* TIFF_API TIFFGetVersion(void);

const TIFFCodec* TIFF_API TIFFFindCODEC(tiff_uint16_t);
TIFFCodec* TIFF_API TIFFRegisterCODEC(tiff_uint16_t, const char*, TIFFInitMethod);
void TIFF_API TIFFUnRegisterCODEC(TIFFCodec*);
tiff_int_t TIFF_API TIFFIsCODECConfigured(tiff_uint16_t);
TIFFCodec* TIFF_API TIFFGetConfiguredCODECs(void);

/*
 * Auxiliary functions.
 */

void* TIFF_API _TIFFmalloc(tmsize_t s);
void* TIFF_API _TIFFcalloc(tmsize_t nmemb, tmsize_t siz);
void* TIFF_API _TIFFrealloc(void* p, tmsize_t s);
void TIFF_API _TIFFmemset(void* p, tiff_int_t v, tmsize_t c);
void TIFF_API _TIFFmemcpy(void* d, const void* s, tmsize_t c);
tiff_int_t TIFF_API _TIFFmemcmp(const void* p1, const void* p2, tmsize_t c);
void TIFF_API _TIFFfree(void* p);

/*
** Stuff, related to tag handling and creating custom tags.
*/
tiff_int_t TIFF_API TIFFGetTagListCount( TIFF * );
uint32 TIFF_API TIFFGetTagListEntry( TIFF *, tiff_int_t tag_index );
    
#define TIFF_ANY       TIFF_NOTYPE     /* for field descriptor searching */
#define TIFF_VARIABLE  -1              /* marker for variable length tags */
#define TIFF_SPP       -2              /* marker for SamplesPerPixel tags */
#define TIFF_VARIABLE2 -3              /* marker for uint32 var-length tags */

#define FIELD_CUSTOM    65

typedef struct _TIFFField TIFFField;
typedef struct _TIFFFieldArray TIFFFieldArray;

const TIFFField* TIFF_API TIFFFindField(TIFF *, uint32, tiff_enum(TIFFDataType));
const TIFFField* TIFF_API TIFFFieldWithTag(TIFF*, uint32);
const TIFFField* TIFF_API TIFFFieldWithName(TIFF*, const char *);

uint32 TIFF_API TIFFFieldTag(const TIFFField*);
const char* TIFF_API TIFFFieldName(const TIFFField*);
TIFFDataType TIFF_API TIFFFieldDataType(const TIFFField*);
tiff_int_t TIFF_API TIFFFieldPassCount(const TIFFField*);
tiff_int_t TIFF_API TIFFFieldReadCount(const TIFFField*);
tiff_int_t TIFF_API TIFFFieldWriteCount(const TIFFField*);

typedef tiff_int_t TIFF_CALLBACK (*TIFFVSetMethod)(TIFF*, uint32, va_list);
typedef tiff_int_t TIFF_CALLBACK (*TIFFVGetMethod)(TIFF*, uint32, va_list);
typedef void TIFF_CALLBACK (*TIFFPrintMethod)(TIFF*, FILE*, long);

typedef struct {
    TIFFVSetMethod vsetfield; /* tag set routine */
    TIFFVGetMethod vgetfield; /* tag get routine */
    TIFFPrintMethod printdir; /* directory print routine */
} TIFFTagMethods;

TIFFTagMethods *TIFF_API TIFFAccessTagMethods(TIFF *);
void *TIFF_API TIFFGetClientInfo(TIFF *, const char *);
void TIFF_API TIFFSetClientInfo(TIFF *, void *, const char *);

void TIFF_API TIFFCleanup(TIFF* tif);
void TIFF_API TIFFClose(TIFF* tif);
tiff_int_t TIFF_API TIFFFlush(TIFF* tif);
tiff_int_t TIFF_API TIFFFlushData(TIFF* tif);
tiff_int_t TIFF_API TIFFGetField(TIFF* tif, ttag_t tag, ...);
tiff_int_t TIFF_API TIFFVGetField(TIFF* tif, ttag_t tag, va_list ap);
tiff_int_t TIFF_API TIFFGetFieldDefaulted(TIFF* tif, ttag_t tag, ...);
tiff_int_t TIFF_API TIFFVGetFieldDefaulted(TIFF* tif, ttag_t tag, va_list ap);
tiff_int_t TIFF_API TIFFReadDirectory(TIFF* tif);
tiff_int_t TIFF_API TIFFReadCustomDirectory(TIFF* tif, toff_t diroff, const TIFFFieldArray* infoarray);
tiff_int_t TIFF_API TIFFReadEXIFDirectory(TIFF* tif, toff_t diroff);
uint64 TIFF_API TIFFScanlineSize64(TIFF* tif);
tmsize_t TIFF_API TIFFScanlineSize(TIFF* tif);
uint64 TIFF_API TIFFRasterScanlineSize64(TIFF* tif);
tmsize_t TIFF_API TIFFRasterScanlineSize(TIFF* tif);
uint64 TIFF_API TIFFStripSize64(TIFF* tif);
tmsize_t TIFF_API TIFFStripSize(TIFF* tif);
uint64 TIFF_API TIFFRawStripSize64(TIFF* tif, uint32 strip);
tmsize_t TIFF_API TIFFRawStripSize(TIFF* tif, uint32 strip);
uint64 TIFF_API TIFFVStripSize64(TIFF* tif, uint32 nrows);
tmsize_t TIFF_API TIFFVStripSize(TIFF* tif, uint32 nrows);
uint64 TIFF_API TIFFTileRowSize64(TIFF* tif);
tmsize_t TIFF_API TIFFTileRowSize(TIFF* tif);
uint64 TIFF_API TIFFTileSize64(TIFF* tif);
tmsize_t TIFF_API TIFFTileSize(TIFF* tif);
uint64 TIFF_API TIFFVTileSize64(TIFF* tif, uint32 nrows);
tmsize_t TIFF_API TIFFVTileSize(TIFF* tif, uint32 nrows);
uint32 TIFF_API TIFFDefaultStripSize(TIFF* tif, uint32 request);
void TIFF_API TIFFDefaultTileSize(TIFF*, uint32*, uint32*);
tiff_int_t TIFF_API TIFFFileno(TIFF*);
tiff_int_t TIFF_API TIFFSetFileno(TIFF*, tiff_int_t);
thandle_t TIFF_API TIFFClientdata(TIFF*);
thandle_t TIFF_API TIFFSetClientdata(TIFF*, thandle_t);
tiff_int_t TIFF_API TIFFGetMode(TIFF*);
tiff_int_t TIFF_API TIFFSetMode(TIFF*, tiff_int_t);
tiff_int_t TIFF_API TIFFIsTiled(TIFF*);
tiff_int_t TIFF_API TIFFIsByteSwapped(TIFF*);
tiff_int_t TIFF_API TIFFIsUpSampled(TIFF*);
tiff_int_t TIFF_API TIFFIsMSB2LSB(TIFF*);
tiff_int_t TIFF_API TIFFIsBigEndian(TIFF*);
TIFFReadWriteProc TIFF_API TIFFGetReadProc(TIFF*);
TIFFReadWriteProc TIFF_API TIFFGetWriteProc(TIFF*);
TIFFSeekProc TIFF_API TIFFGetSeekProc(TIFF*);                                                          
TIFFCloseProc TIFF_API TIFFGetCloseProc(TIFF*);
TIFFSizeProc TIFF_API TIFFGetSizeProc(TIFF*);
TIFFMapFileProc TIFF_API TIFFGetMapFileProc(TIFF*);
TIFFUnmapFileProc TIFF_API TIFFGetUnmapFileProc(TIFF*);
uint32 TIFF_API TIFFCurrentRow(TIFF*);
uint16 TIFF_API TIFFCurrentDirectory(TIFF*);
uint16 TIFF_API TIFFNumberOfDirectories(TIFF*);
uint64 TIFF_API TIFFCurrentDirOffset(TIFF*);
uint32 TIFF_API TIFFCurrentStrip(TIFF*);
uint32 TIFF_API TIFFCurrentTile(TIFF* tif);
tiff_int_t TIFF_API TIFFReadBufferSetup(TIFF* tif, void* bp, tmsize_t size);
tiff_int_t TIFF_API TIFFWriteBufferSetup(TIFF* tif, void* bp, tmsize_t size);  
tiff_int_t TIFF_API TIFFSetupStrips(TIFF *);
tiff_int_t TIFF_API TIFFWriteCheck(TIFF*, tiff_int_t, const char *);
void TIFF_API TIFFFreeDirectory(TIFF*);
tiff_int_t TIFF_API TIFFCreateDirectory(TIFF*);
tiff_int_t TIFF_API TIFFCreateCustomDirectory(TIFF*,const TIFFFieldArray*);
tiff_int_t TIFF_API TIFFCreateEXIFDirectory(TIFF*);
tiff_int_t TIFF_API TIFFLastDirectory(TIFF*);
tiff_int_t TIFF_API TIFFSetDirectory(TIFF*, tiff_uint16_t);
tiff_int_t TIFF_API TIFFSetSubDirectory(TIFF*, uint64);
tiff_int_t TIFF_API TIFFUnlinkDirectory(TIFF*, tiff_uint16_t);
tiff_int_t TIFF_API TIFFSetField(TIFF*, ttag_t, ...);
tiff_int_t TIFF_API TIFFVSetField(TIFF*, ttag_t, va_list);
tiff_int_t TIFF_API TIFFUnsetField(TIFF*, ttag_t);
tiff_int_t TIFF_API TIFFWriteDirectory(TIFF *);
tiff_int_t TIFF_API TIFFWriteCustomDirectory(TIFF *, uint64 *);
tiff_int_t TIFF_API TIFFCheckpointDirectory(TIFF *);
tiff_int_t TIFF_API TIFFRewriteDirectory(TIFF *);

#if defined(c_plusplus) || defined(__cplusplus)
void TIFF_API TIFFPrintDirectory(TIFF*, FILE*, long = 0);
tiff_int_t TIFF_API TIFFReadScanline(TIFF* tif, void* buf, uint32 row, tiff_uint16_t sample = 0);
tiff_int_t TIFF_API TIFFWriteScanline(TIFF* tif, void* buf, uint32 row, tiff_uint16_t sample = 0);
tiff_int_t TIFF_API TIFFReadRGBAImage(TIFF*, uint32, uint32, uint32*, tiff_int_t = 0);
tiff_int_t TIFF_API TIFFReadRGBAImageOriented(TIFF*, uint32, uint32, uint32*, tiff_int_t = ORIENTATION_BOTLEFT, tiff_int_t = 0);
#else
void TIFF_API TIFFPrintDirectory(TIFF*, FILE*, long);
tiff_int_t TIFF_API TIFFReadScanline(TIFF* tif, void* buf, uint32 row, tiff_uint16_t sample);
tiff_int_t TIFF_API TIFFWriteScanline(TIFF* tif, void* buf, uint32 row, tiff_uint16_t sample);
tiff_int_t TIFF_API TIFFReadRGBAImage(TIFF*, uint32, uint32, uint32*, tiff_int_t);
tiff_int_t TIFF_API TIFFReadRGBAImageOriented(TIFF*, uint32, uint32, uint32*, tiff_int_t, tiff_int_t);
#endif

tiff_int_t TIFF_API TIFFReadRGBAStrip(TIFF*, uint32, uint32 * );
tiff_int_t TIFF_API TIFFReadRGBATile(TIFF*, uint32, uint32, uint32 * );
tiff_int_t TIFF_API TIFFReadRGBAStripExt(TIFF*, uint32, uint32 *, tiff_int_t stop_on_error );
tiff_int_t TIFF_API TIFFReadRGBATileExt(TIFF*, uint32, uint32, uint32 *, tiff_int_t stop_on_error );
tiff_int_t TIFF_API TIFFRGBAImageOK(TIFF*, char [1024]);
tiff_int_t TIFF_API TIFFRGBAImageBegin(TIFFRGBAImage*, TIFF*, tiff_int_t, char [1024]);
tiff_int_t TIFF_API TIFFRGBAImageGet(TIFFRGBAImage*, uint32*, uint32, uint32);
void TIFF_API TIFFRGBAImageEnd(TIFFRGBAImage*);
TIFF* TIFF_API TIFFOpen(const char*, const char*);
# ifdef __WIN32__
TIFF* TIFF_API TIFFOpenW(const wchar_t*, const char*);
# endif /* __WIN32__ */
TIFF* TIFF_API TIFFFdOpen(tiff_int_t, const char*, const char*);
TIFF* TIFF_API TIFFClientOpen(const char*, const char*, thandle_t, TIFFReadWriteProc, TIFFReadWriteProc, TIFFSeekProc, TIFFCloseProc, TIFFSizeProc, TIFFMapFileProc, TIFFUnmapFileProc);
const char* TIFF_API TIFFFileName(TIFF*);
const char* TIFF_API TIFFSetFileName(TIFF*, const char *);
void TIFF_API TIFFError(const char*, const char*, ...) __attribute__((__format__ (__printf__,2,3)));
void TIFF_API TIFFErrorExt(thandle_t, const char*, const char*, ...) __attribute__((__format__ (__printf__,3,4)));
void TIFF_API TIFFWarning(const char*, const char*, ...) __attribute__((__format__ (__printf__,2,3)));
void TIFF_API TIFFWarningExt(thandle_t, const char*, const char*, ...) __attribute__((__format__ (__printf__,3,4)));
TIFFErrorHandler TIFF_API TIFFSetErrorHandler(TIFFErrorHandler);
TIFFErrorHandlerExt TIFF_API TIFFSetErrorHandlerExt(TIFFErrorHandlerExt);
TIFFErrorHandler TIFF_API TIFFSetWarningHandler(TIFFErrorHandler);
TIFFErrorHandlerExt TIFF_API TIFFSetWarningHandlerExt(TIFFErrorHandlerExt);
TIFFExtendProc TIFF_API TIFFSetTagExtender(TIFFExtendProc);
uint32 TIFF_API TIFFComputeTile(TIFF* tif, uint32 x, uint32 y, uint32 z, tiff_uint16_t s);
tiff_int_t TIFF_API TIFFCheckTile(TIFF* tif, uint32 x, uint32 y, uint32 z, tiff_uint16_t s);
uint32 TIFF_API TIFFNumberOfTiles(TIFF*);
tmsize_t TIFF_API TIFFReadTile(TIFF* tif, void* buf, uint32 x, uint32 y, uint32 z, tiff_uint16_t s);
tmsize_t TIFF_API TIFFWriteTile(TIFF* tif, void* buf, uint32 x, uint32 y, uint32 z, tiff_uint16_t s);
uint32 TIFF_API TIFFComputeStrip(TIFF*, uint32, tiff_uint16_t);
uint32 TIFF_API TIFFNumberOfStrips(TIFF*);
tmsize_t TIFF_API TIFFReadEncodedStrip(TIFF* tif, uint32 strip, void* buf, tmsize_t size);
tmsize_t TIFF_API TIFFReadRawStrip(TIFF* tif, uint32 strip, void* buf, tmsize_t size);  
tmsize_t TIFF_API TIFFReadEncodedTile(TIFF* tif, uint32 tile, void* buf, tmsize_t size);  
tmsize_t TIFF_API TIFFReadRawTile(TIFF* tif, uint32 tile, void* buf, tmsize_t size);  
tmsize_t TIFF_API TIFFWriteEncodedStrip(TIFF* tif, uint32 strip, void* data, tmsize_t cc);
tmsize_t TIFF_API TIFFWriteRawStrip(TIFF* tif, uint32 strip, void* data, tmsize_t cc);  
tmsize_t TIFF_API TIFFWriteEncodedTile(TIFF* tif, uint32 tile, void* data, tmsize_t cc);  
tmsize_t TIFF_API TIFFWriteRawTile(TIFF* tif, uint32 tile, void* data, tmsize_t cc);  
tiff_int_t TIFF_API TIFFDataWidth(tiff_enum(TIFFDataType));    /* table of tag datatype widths */
void TIFF_API TIFFSetWriteOffset(TIFF* tif, toff_t off);
void TIFF_API TIFFSwabShort(uint16*);
void TIFF_API TIFFSwabLong(uint32*);
void TIFF_API TIFFSwabLong8(uint64*);
void TIFF_API TIFFSwabFloat(__gcc_float_t*);
void TIFF_API TIFFSwabDouble(__gcc_double_t*);
void TIFF_API TIFFSwabArrayOfShort(uint16* wp, tmsize_t n);
void TIFF_API TIFFSwabArrayOfTriples(uint8* tp, tmsize_t n);
void TIFF_API TIFFSwabArrayOfLong(uint32* lp, tmsize_t n);
void TIFF_API TIFFSwabArrayOfLong8(uint64* lp, tmsize_t n);
void TIFF_API TIFFSwabArrayOfFloat(__gcc_float_t* fp, tmsize_t n);
void TIFF_API TIFFSwabArrayOfDouble(__gcc_double_t* dp, tmsize_t n);
void TIFF_API TIFFReverseBits(uint8* cp, tmsize_t n);
const unsigned char* TIFF_API TIFFGetBitRevTable(tiff_int_t);

#if LOGLUV_PUBLIC
#define U_NEU		0.210526316
#define V_NEU		0.473684211
#define UVSCALE		410.
double TIFF_API LogL16toY(tiff_int_t);
double TIFF_API LogL10toY(tiff_int_t);
void TIFF_API XYZtoRGB24(float*, uint8*);
tiff_int_t TIFF_API uv_decode(double*, double*, tiff_int_t);
void TIFF_API LogLuv24toXYZ(uint32, float*);
void TIFF_API LogLuv32toXYZ(uint32, float*);
#if defined(c_plusplus) || defined(__cplusplus)
tiff_int_t TIFF_API LogL16fromY(double, tiff_int_t = SGILOGENCODE_NODITHER);
tiff_int_t TIFF_API LogL10fromY(double, tiff_int_t = SGILOGENCODE_NODITHER);
tiff_int_t TIFF_API uv_encode(double, double, tiff_int_t = SGILOGENCODE_NODITHER);
uint32 TIFF_API LogLuv24fromXYZ(float*, tiff_int_t = SGILOGENCODE_NODITHER);
uint32 TIFF_API LogLuv32fromXYZ(float*, tiff_int_t = SGILOGENCODE_NODITHER);
#else
tiff_int_t TIFF_API LogL16fromY(double, tiff_int_t);
tiff_int_t TIFF_API LogL10fromY(double, tiff_int_t);
tiff_int_t TIFF_API uv_encode(double, double, tiff_int_t);
uint32 TIFF_API LogLuv24fromXYZ(float*, tiff_int_t);
uint32 TIFF_API LogLuv32fromXYZ(float*, tiff_int_t);
#endif
#endif /* LOGLUV_PUBLIC */

tiff_int_t TIFF_API TIFFCIELabToRGBInit(TIFFCIELabToRGB*, const TIFFDisplay *, float*);
void TIFF_API TIFFCIELabToXYZ(TIFFCIELabToRGB *, uint32, int32, int32, float *, float *, float *);
void TIFF_API TIFFXYZToRGB(TIFFCIELabToRGB *, __gcc_float_t, __gcc_float_t, __gcc_float_t, uint32 *, uint32 *, uint32 *);

tiff_int_t TIFF_API TIFFYCbCrToRGBInit(TIFFYCbCrToRGB*, float*, float*);
void TIFF_API TIFFYCbCrtoRGB(TIFFYCbCrToRGB *, uint32, int32, int32, uint32 *, uint32 *, uint32 *);

const char *TIFF_API _TIFF_uint64_to_str(ll_uint64_t value, char *buf, tiff_int_t issigned);

/****************************************************************************
 *               O B S O L E T E D    I N T E R F A C E S
 *
 * Don't use this stuff in your applications, it may be removed in the future
 * libtiff versions.
 ****************************************************************************/
typedef	struct {
	ttag_t	field_tag;		/* field's tag */
	short	field_readcount;	/* read count/TIFF_VARIABLE/TIFF_SPP */
	short	field_writecount;	/* write count/TIFF_VARIABLE */
	tiff_enum(TIFFDataType) field_type;	/* type of associated data */
    unsigned short field_bit;	/* bit in fieldsset bit vector */
	unsigned char field_oktochange;	/* if true, can change while writing */
	unsigned char field_passcount;	/* if true, pass dir count on set */
	char	*field_name;		/* ASCII name */
} TIFFFieldInfo;

tiff_int_t TIFF_API TIFFMergeFieldInfo(TIFF*, const TIFFFieldInfo[], uint32);
        
#if defined(c_plusplus) || defined(__cplusplus)
}
#endif

#if defined(TIFF_SLB) && (defined(__atarist__) || defined(__TOS__))
#include <slb/tiff.h>
#endif

#if defined(__PUREC__) && !defined(BUILDING_LIBTIFF) && !defined(TIFF_TOOL_HACKS)
struct _TIFFField { int dummy; };
struct _TIFFFieldArray { int dummy; };
struct tiff { int dummy; };
#endif

#endif /* _TIFFIO_ */
